/*
 * General functionality supporting file notifications.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "file_notifier_registry.h"



FileNotifierRegistry::FileNotifierRegistry()
{
}



/* Obtain a file-specific notification manager. */

FileNotification *FileNotifierRegistry::_get(fileid_t fileid)
{
    FileNotifiers::iterator entry = _notifiers.find(fileid);

    if (entry != _notifiers.end())
        return _notifiers[fileid];
    else
        return NULL;
}

/* Register a notification manager for the given 'fileid'. */

void FileNotifierRegistry::attach(fileid_t fileid)
{
    std::lock_guard<std::mutex> guard(_lock);

    FileNotification *notifier = _get(fileid);

    if (notifier == NULL)
    {
        notifier = new FileNotification;
        _notifiers[fileid] = notifier;
    }

    notifier->attach();
}

/* Detach from and potentially remove a notification manager for the given
   'fileid'. */

void FileNotifierRegistry::detach(fileid_t fileid)
{
    std::lock_guard<std::mutex> guard(_lock);

    FileNotification *notifier = _get(fileid);

    if ((notifier != NULL) && (!notifier->detach()))
    {
        _notifiers.erase(fileid);
        delete notifier;
    }
}

/* Obtain a file-specific notification manager. */

FileNotification *FileNotifierRegistry::get(fileid_t fileid)
{
    std::lock_guard<std::mutex> guard(_lock);

    return _get(fileid);
}

// vim: tabstop=4 expandtab shiftwidth=4
